#!/bin/bash
# Copyright © Huawei Technologies Co., Ltd. 2022-2025. All rights reserved.
##############################################################
#  The entry script of whitebox
#
## @Filename:white_box.shdG
##############################################################
set -e

PACKAGE_UNZIP_PATH="/run/whitebox_tmp"
OM_SERVICES=("om-init" "platform-app" "ibma-edge-start" "start-nginx.service")
OM_WORK_DIR="/usr/local/mindx/MindXOM"
WHITEBOX_BACK_DIR="/home/data/WhiteboxConfig"
OM_WHITEBOX_PATH="${OM_WORK_DIR}/software/nginx/html/manager/WhiteboxConfig"
WHITEBOX_JSON_DIR="json"
WHITEBOX_IMG_DIR="img"
WHITEBOX_JS_PATH="${WHITEBOX_JSON_DIR}/config.json"
WHITEBOX_IMG_LIST_PATH=("${WHITEBOX_IMG_DIR}/device.png" "${WHITEBOX_IMG_DIR}/favicon.ico" \
                        "${WHITEBOX_IMG_DIR}/model.svg" "${WHITEBOX_IMG_DIR}/logo.png" "${WHITEBOX_IMG_DIR}/login.png")
WHITEBOX_IMG_EN_PATH="${WHITEBOX_IMG_DIR}/device-en.png"
P1_DEV_NAME="/dev/mmcblk0p1"
GOLDEN_MOUNT_PATH="/mnt/p1"
GOLDEN_WHITEBOX_CONFIG_PATH="${GOLDEN_MOUNT_PATH}/whiteboxConfig"
PKI_CERT_PATH="${GOLDEN_MOUNT_PATH}/om_certs"
CMS_VERIFY_FILE="${OM_WORK_DIR}/software/ibma/common/verify_cms_file.py"
CMS_LIBVERIFY_SO="${OM_WORK_DIR}/lib/libverify.so"
PACKAGE_VERCFG_XML="${PACKAGE_UNZIP_PATH}/vercfg.xml"
PACKAGE_VERSION_XML="${PACKAGE_UNZIP_PATH}/version.xml"
OM_CURR_VERSION_XML="${OM_WORK_DIR}/version.xml"

# 文件大小限制最大10M
MAX_SIZE="10485760"

# 输出参数
PACKAGE_PATH=""
INPUT_WHITE_BOX_CONFIG_PATH=""

#***********************************************
# Description: 获取系统盘的主分区的盘符路径
# Parameter:
#   input:  NA
#   output: 系统盘的主分区的盘符路径
# Return Value : NA
#***********************************************
function get_sys_main_dev_path()
{
    local sys_main_dev=""
    sys_main_dev=$(grep "root=" /proc/cmdline | sed 's/.*\sroot=//g' | awk -F" " '{print $1}')
    if [[ "${sys_main_dev}" =~ ^PARTUUID=[0-9a-f\-]{36,36}$ ]]; then
        sys_main_dev=$(blkid -t "${sys_main_dev}" | awk -F: '{print $1}')
    fi
    echo "${sys_main_dev}"
}

#***********************************************
# Description: 判断当前系统是否从emmc启动
# Parameter:
#   input:  NA
#   output: NA
# Return Value: 0 -- 是
#               1 -- m.2
#***********************************************
function check_boot_from_emmc()
{
    local sys_main_dev=""
    sys_main_dev=$(get_sys_main_dev_path)
    if [[ "${sys_main_dev}" =~ ^/dev/mmcblk0p[0-9]+$ ]]; then
        return 0
    fi
    return 1
}


function xml_verify()
{
    # 校验文件的sha256值
    for file in $(find "${PACKAGE_UNZIP_PATH}" -maxdepth 1 -type f | grep "A500-A2-os_.\{1,\}.tar.gz"); do
        if [[ $(sha256sum "${file}" | awk '{print $1}') != $(sed -n "/<FilePath>$(basename "${file}")<\/FilePath>/{n;s|^.*<SHAValue>\(.*\)</SHAValue>$|\1|;p}" "${PACKAGE_VERCFG_XML}") ]]; then
            echo "${file} sha256 is not matched"
            return 1
        fi
    done

    for file in $(find "${PACKAGE_UNZIP_PATH}" -maxdepth 1 -type f | grep "Ascend-mindxedge-om_.\{1,\}.tar.gz"); do
        if [[ $(sha256sum "${file}" | awk '{print $1}') != $(sed -n "/<FilePath>$(basename "${file}")<\/FilePath>/{n;s|^.*<SHAValue>\(.*\)</SHAValue>$|\1|;p}" "${PACKAGE_VERCFG_XML}") ]]; then
            echo "${file} sha256 is not matched"
            return 1
        fi
    done

    for file in $(find "${PACKAGE_UNZIP_PATH}" -maxdepth 1 -type f | grep "Ascend-mindxedge-mefedge_.\{1,\}.tar.gz"); do
        if [[ $(sha256sum "${file}" | awk '{print $1}') != $(sed -n "/<FilePath>$(basename "${file}")<\/FilePath>/{n;s|^.*<SHAValue>\(.*\)</SHAValue>$|\1|;p}" "${PACKAGE_VERCFG_XML}") ]]; then
            echo "${file} sha256 is not matched"
            return 1
        fi
    done

    for file in $(find "${PACKAGE_UNZIP_PATH}" -maxdepth 1 -type f | grep "Ascend-mindx-toolbox_.\{1,\}.tar.gz"); do
        if [[ $(sha256sum "${file}" | awk '{print $1}') != $(sed -n "/<FilePath>$(basename "${file}")<\/FilePath>/{n;s|^.*<SHAValue>\(.*\)</SHAValue>$|\1|;p}" "${PACKAGE_VERCFG_XML}") ]]; then
            echo "${file} sha256 is not matched"
            return 1
        fi
    done

    for file in $(find "${PACKAGE_UNZIP_PATH}" -maxdepth 1 -type f | grep "Ascend-cann-nnrt_.\{1,\}.tar.gz"); do
        if [[ $(sha256sum "${file}" | awk '{print $1}') != $(sed -n "/<FilePath>$(basename "${file}")<\/FilePath>/{n;s|^.*<SHAValue>\(.*\)</SHAValue>$|\1|;p}" "${PACKAGE_VERCFG_XML}") ]]; then
            echo "${file} sha256 is not matched"
            return 1
        fi
    done

    if [[ $(sha256sum "${PACKAGE_VERSION_XML}" | awk '{print $1}') != $(sed -n "/<FilePath>$(basename "${PACKAGE_VERSION_XML}")<\/FilePath>/{n;s|^.*<SHAValue>\(.*\)</SHAValue>$|\1|;p}" "${PACKAGE_VERCFG_XML}") ]]; then
        echo "$(basename "${PACKAGE_VERSION_XML}") sha256 is not matched"
        return 1
    fi

    # 校验文件的版本号是否匹配
    if [[ $(sed -n "s|.*<Version>\(.*\)</Version>|\1|p" "${PACKAGE_VERSION_XML}") != $(sed -n "s|.*<FirmwareVersion>\(.*\)</FirmwareVersion>|\1|p" "${OM_CURR_VERSION_XML}") ]]; then
        echo "$(basename "${PACKAGE_VERSION_XML}") version is not matched"
        return 1
    fi
    return 0
}

function check_png_img()
{
    local image_path="$1"
    local file_attr=""
    local img_suffix=""

    img_suffix="$(basename "${image_path}" | awk -F "." '{print $2}')"
    # 检视是否为普通文件或文件是否存在
    if [[ ! -f "${image_path}" ]]; then
        echo "${img_suffix} image ${image_path} is not valid"
        return 1
    fi

    # 检查是否为root属主
    if [[ $(stat -c %U "${image_path}") != "root" ]]; then
        echo "${image_path} is not root owner, please check"
        return 1
    fi

    # 检查文件大小
    if [[ $(stat -c %s "${image_path}") -gt "${MAX_SIZE}" ]]; then
        echo "${img_suffix} image ${image_path} is too large"
        return 1
    fi

    # 检查文件属性
    file_attr=$(file -b "${image_path}")

    # svg图片
    if [[ "${img_suffix}" == "svg" ]]; then
        if [[ "$(echo "${file_attr}" | awk -F "," '{if($1~"SVG") print "0"; else print "1"}')" != "0" ]]; then
            echo "${img_suffix} image ${image_path} check file type failed"
            return 1
        fi
        echo "check ${img_suffix} image ${image_path} success"
        return 0
    fi
    
    # png图片
    if [[ "${img_suffix}" == "png" ]]; then
        if [[ "$(echo "${file_attr}" | awk -F "," '{if($1~"PNG image") print "0"; else print "1"}')" != "0" ]]; then
            echo "${img_suffix} image ${image_path} check file type failed"
            return 1
        fi

    # ico图片
    elif [[ "${img_suffix}" == "ico" ]]; then
        if [[ "$(echo "${file_attr}" | awk -F "," '{if($1~"icon") print "0"; else print "1"}')" != "0" ]]; then
            echo "${img_suffix} image ${image_path} check file type failed"
            return 1
        fi
    fi

    echo "check ${img_suffix} image ${image_path} success"
    return 0
}

function check_js_cfg()
{
    if [[ ! -f "${INPUT_WHITE_BOX_CONFIG_PATH}/${WHITEBOX_JS_PATH}" ]]; then
        echo "${INPUT_WHITE_BOX_CONFIG_PATH}/${WHITEBOX_JS_PATH} is not valid"
        return 1
    fi

    if [[ $(stat -c %U "${INPUT_WHITE_BOX_CONFIG_PATH}/${WHITEBOX_JS_PATH}") != "root" ]]; then
        echo "${INPUT_WHITE_BOX_CONFIG_PATH}/${WHITEBOX_JS_PATH} is not root owner, please check"
        return 1
    fi

    if [[ $(stat -c %s "${INPUT_WHITE_BOX_CONFIG_PATH}/${WHITEBOX_JS_PATH}") -gt "${MAX_SIZE}" ]]; then
        echo "${INPUT_WHITE_BOX_CONFIG_PATH}/${WHITEBOX_JS_PATH} is too large"
        return 1
    fi

    if ! python3 -m json.tool "${INPUT_WHITE_BOX_CONFIG_PATH}/${WHITEBOX_JS_PATH}" &> /dev/null; then
        echo "Invalid json ${INPUT_WHITE_BOX_CONFIG_PATH}/${WHITEBOX_JS_PATH}"
        return 1
    fi

    if ! grep -P '^\s*"model"\s*:\s*"[a-zA-Z0-9()\ ]{1,128}"' "${INPUT_WHITE_BOX_CONFIG_PATH}/${WHITEBOX_JS_PATH}" &> /dev/null; then
        echo "check ${INPUT_WHITE_BOX_CONFIG_PATH}/${WHITEBOX_JS_PATH} failed, please check <model> field"
        return 1
    fi

    if ! grep -P '^\s*"systemName"\s*:\s*{' "${INPUT_WHITE_BOX_CONFIG_PATH}/${WHITEBOX_JS_PATH}" &> /dev/null; then
        echo "check ${INPUT_WHITE_BOX_CONFIG_PATH}/${WHITEBOX_JS_PATH} failed, please check <systemName> field"
        return 1
    fi

    if ! grep -P '^\s*"websiteTitle"\s*:\s*{' "${INPUT_WHITE_BOX_CONFIG_PATH}/${WHITEBOX_JS_PATH}" &> /dev/null; then
        echo "check ${INPUT_WHITE_BOX_CONFIG_PATH}/${WHITEBOX_JS_PATH} failed, please check <websiteTitle> field"
        return 1
    fi

    if ! grep -P '^\s*"userGuide"\s*:\s*{' "${INPUT_WHITE_BOX_CONFIG_PATH}/${WHITEBOX_JS_PATH}" &> /dev/null; then
        echo "check ${INPUT_WHITE_BOX_CONFIG_PATH}/${WHITEBOX_JS_PATH} failed, please check <userGuide> field"
        return 1
    fi

    if ! grep -P '^\s*"copyRight"\s*:\s*{' "${INPUT_WHITE_BOX_CONFIG_PATH}/${WHITEBOX_JS_PATH}" &> /dev/null; then
        echo "check ${INPUT_WHITE_BOX_CONFIG_PATH}/${WHITEBOX_JS_PATH} failed, please check <copyRight> field"
        return 1
    fi

    if [[ $(grep -cP '^\s*"zh"\s*:\s*"[a-zA-Z0-9()\x{4e00}-\x{9fa5}\x{FF0C}\x{3002}\x{FF08}\x{FF09} ]{1,128}"' "${INPUT_WHITE_BOX_CONFIG_PATH}/${WHITEBOX_JS_PATH}") -ne "4" ]]; then
        echo "check ${INPUT_WHITE_BOX_CONFIG_PATH}/${WHITEBOX_JS_PATH} failed, missing <zh> field, please check "
        return 1
    fi

    if [[ $(grep -cP '^\s*"en"\s*:\s*"[a-zA-Z0-9()., ]{1,128}"' "${INPUT_WHITE_BOX_CONFIG_PATH}/${WHITEBOX_JS_PATH}") -ne "4" ]]; then
        echo "check ${INPUT_WHITE_BOX_CONFIG_PATH}/${WHITEBOX_JS_PATH} failed, missing <en> field, please check "
        return 1
    fi

    echo "check ${INPUT_WHITE_BOX_CONFIG_PATH}/${WHITEBOX_JS_PATH} success"
    return 0
}

function whitebox_cfg_verify()
{
    for (( i= 0; i < "${#WHITEBOX_IMG_LIST_PATH[@]}"; i++)); do
        if ! check_png_img "${INPUT_WHITE_BOX_CONFIG_PATH}/${WHITEBOX_IMG_LIST_PATH[i]}"; then
            echo "check whitebox config img failed, please check"
            return 1
        fi
    done

    if [[ -f "${INPUT_WHITE_BOX_CONFIG_PATH}/${WHITEBOX_IMG_EN_PATH}" ]]; then
        if ! check_png_img "${INPUT_WHITE_BOX_CONFIG_PATH}/${WHITEBOX_IMG_EN_PATH}"; then
            echo "check whitebox config img failed, please check"
            return 1
        fi
    fi

    if ! check_js_cfg; then
        echo "check whitebox config js failed, please check"
        return 1
    fi

    return 0
}

function check_whitebox_config()
{
    if check_boot_from_emmc; then
        rm -rf "${PACKAGE_UNZIP_PATH}"
        mkdir "${PACKAGE_UNZIP_PATH}"
        if ! unzip -oq "${PACKAGE_PATH}" -d "${PACKAGE_UNZIP_PATH}"; then
            echo "unzip ${PACKAGE_PATH} failed, please check"
            return 1
        fi

        # 签名校验
        (
            export LD_LIBRARY_PATH="${OM_WORK_DIR}"/lib:"${LD_LIBRARY_PATH}"
            export PYTHONPATH="${OM_WORK_DIR}"/software/ibma:"${OM_WORK_DIR}"/scripts/python
            python3 -u "${CMS_VERIFY_FILE}" "${CMS_LIBVERIFY_SO}" "${PACKAGE_VERCFG_XML}.cms" "${PACKAGE_VERCFG_XML}.crl" "${PACKAGE_VERCFG_XML}"
        )
        local ret=$?
        if [[ "${ret}" -ne 0 ]]; then
            echo "${PACKAGE_PATH} cms verify failed, please check"
            return 1
        fi

        # 校验xml文件
        if ! xml_verify; then
            echo "sha256 verify failed, please check"
            return 1
        fi
    fi
    echo "check cms signature success"

    # 白牌文件匹配
    if ! whitebox_cfg_verify; then
        echo "whitebox config verify failed, please check"
        return 1
    fi

    return 0
}

function check_param_env()
{
    # 校验操作用户
    if [[ $(whoami) != "root" ]]; then
        echo "The current login user is not root, please check"
        return 1
    fi
    echo "check operate user success"

    # 校验输入参数个数
    if [[ -z "$1" || -z "$2" ]]; then
        echo "check param failed, whitebox.sh need two param: <A500-A2-firmware zip path> and <whitebox config path>."
        return 1
    fi

    # 校验输入参数路径
    PACKAGE_PATH=$(realpath "$1")
    INPUT_WHITE_BOX_CONFIG_PATH=$(realpath "$2")
    if [[ ! -f "${PACKAGE_PATH}" ]]; then
        echo "the firmware package is not exits, please check"
        return 1
    fi

    if [[ $(stat -c %U "${PACKAGE_PATH}") != "root" ]]; then
        echo "the firmware package is not root owner, please check"
        return 1
    fi

    if [[ ! -d "${INPUT_WHITE_BOX_CONFIG_PATH}" ]]; then
        echo "the whitebox config dir is not exits, please check"
        return 1
    fi

    if [[ $(stat -c %U "${INPUT_WHITE_BOX_CONFIG_PATH}") != "root" ]]; then
        echo "the whitebox config dir is not root owner, please check"
        return 1
    fi
    echo "check input param success"

    # 校验白牌配置
    if ! check_whitebox_config; then
        echo "the whitebox config does not match the firmware package, please check"
        return 1
    fi
    echo "check whitebox config success"
    return 0
}

function copy_whitebox_config()
{
    local src_path="$1"
    local dest_path="$2"

    # json配置
    if [[ ! -d "${dest_path}/${WHITEBOX_JSON_DIR}" ]]; then
        mkdir "${dest_path}/${WHITEBOX_JSON_DIR}"
    fi
    if [[ -f "${dest_path}/${WHITEBOX_JS_PATH}" || -L "${dest_path}/${WHITEBOX_JS_PATH}" ]]; then
        unlink "${dest_path}/${WHITEBOX_JS_PATH}"
    fi
    if ! cp "${src_path}/${WHITEBOX_JS_PATH}" "${dest_path}/${WHITEBOX_JS_PATH}"; then
        echo "cp ${WHITEBOX_JS_PATH} failed, please check"
        return 1
    fi

    # 中文图片
    if [[ ! -d "${dest_path}/${WHITEBOX_IMG_DIR}" ]]; then
        mkdir "${dest_path}/${WHITEBOX_IMG_DIR}"
    fi

    for (( i= 0; i < "${#WHITEBOX_IMG_LIST_PATH[@]}"; i++)); do
        if [[ -f "${dest_path}/${WHITEBOX_IMG_LIST_PATH[i]}" || -L "${dest_path}/${WHITEBOX_IMG_LIST_PATH[i]}" ]]; then
            unlink "${dest_path}/${WHITEBOX_IMG_LIST_PATH[i]}"
        fi
        if ! cp "${src_path}/${WHITEBOX_IMG_LIST_PATH[i]}" "${dest_path}/${WHITEBOX_IMG_LIST_PATH[i]}"; then
            echo "cp ${WHITEBOX_IMG_LIST_PATH[i]} failed, please check"
            return 1
        fi
    done

    # 英文图片
    if [[ -f "${src_path}/${WHITEBOX_IMG_EN_PATH}" ]]; then
        if [[ -f "${dest_path}/${WHITEBOX_IMG_EN_PATH}" || -L "${dest_path}/${WHITEBOX_IMG_EN_PATH}" ]]; then
            unlink "${dest_path}/${WHITEBOX_IMG_EN_PATH}"
        fi
        if ! cp "${src_path}/${WHITEBOX_IMG_EN_PATH}" "${dest_path}/${WHITEBOX_IMG_EN_PATH}"; then
            echo "cp ${WHITEBOX_IMG_EN_PATH} failed, please check"
            return 1
        fi
    else
        if [[ -f "${dest_path}/${WHITEBOX_IMG_EN_PATH}" || -L "${dest_path}/${WHITEBOX_IMG_EN_PATH}" ]]; then
            unlink "${dest_path}/${WHITEBOX_IMG_EN_PATH}"
        fi
        if ! cp "${src_path}/${WHITEBOX_IMG_LIST_PATH[0]}" "${dest_path}/${WHITEBOX_IMG_EN_PATH}"; then
            echo "cp ${WHITEBOX_IMG_LIST_PATH[0]} failed, please check"
            return 1
        fi
    fi

    return 0
}


function copy_files_to_p1_part()
{
    if ! blockdev --setrw "${P1_DEV_NAME}"; then
        echo "blockdev --setrw ${P1_DEV_NAME} failed, please check"
        return 1
    fi

    if ! mount -t ext4 "${P1_DEV_NAME}" "${GOLDEN_MOUNT_PATH}"; then
        echo "mount ${P1_DEV_NAME} failed, please check"
        return 1
    fi

    rm -f "$(find "${GOLDEN_MOUNT_PATH}" -maxdepth 1 -type f -name "A500-A2-os_*.tar.gz")"
    if ! cp "$(find "${PACKAGE_UNZIP_PATH}" -maxdepth 1 -type f -name "A500-A2-os_*.tar.gz")" "${GOLDEN_MOUNT_PATH}"; then
        echo "cp ${PACKAGE_PATH} failed, please check"
        return 1
    fi

    rm -f "$(find "${GOLDEN_MOUNT_PATH}" -maxdepth 1 -type f -name "Ascend-mindxedge-om_*.tar.gz")"
    if ! cp "$(find "${PACKAGE_UNZIP_PATH}" -maxdepth 1 -type f -name "Ascend-mindxedge-om_*.tar.gz")" "${GOLDEN_MOUNT_PATH}"; then
        echo "cp ${PACKAGE_PATH} failed, please check"
        return 1
    fi

    rm -f "$(find "${GOLDEN_MOUNT_PATH}" -maxdepth 1 -type f -name "Ascend-mindxedge-mefedge_*.tar.gz")"
    if ! cp "$(find "${PACKAGE_UNZIP_PATH}" -maxdepth 1 -type f -name "Ascend-mindxedge-mefedge_*.tar.gz")" "${GOLDEN_MOUNT_PATH}"; then
        echo "cp ${PACKAGE_PATH} failed, please check"
        return 1
    fi

    rm -f "$(find "${GOLDEN_MOUNT_PATH}" -maxdepth 1 -type f -name "Ascend-mindx-toolbox_*.tar.gz")"
    if ! cp "$(find "${PACKAGE_UNZIP_PATH}" -maxdepth 1 -type f -name "Ascend-mindx-toolbox_*.tar.gz")" "${GOLDEN_MOUNT_PATH}"; then
        echo "cp ${PACKAGE_PATH} failed, please check"
        return 1
    fi

    rm -f "$(find "${GOLDEN_MOUNT_PATH}" -maxdepth 1 -type f -name "Ascend-cann-nnrt_*.tar.gz")"
    if ! cp "$(find "${PACKAGE_UNZIP_PATH}" -maxdepth 1 -type f -name "Ascend-cann-nnrt_*.tar.gz")" "${GOLDEN_MOUNT_PATH}"; then
        echo "cp ${PACKAGE_PATH} failed, please check"
        return 1
    fi

    rm -f "$(find "${GOLDEN_MOUNT_PATH}" -maxdepth 1 -type f -name "reset_om.sh")"
    local om_unpack_path="${PACKAGE_UNZIP_PATH}/unpack_om"
    rm -rf "${om_unpack_path}"
    mkdir "${om_unpack_path}"
    if ! tar -zxf "$(find "${PACKAGE_UNZIP_PATH}" -maxdepth 1 -type f -name "Ascend-mindxedge-om_*.tar.gz")" -C "${om_unpack_path}" &> /dev/null; then
        echo "tar om package failed, please check"
        return 1
    fi
    if ! cp "$(find "${om_unpack_path}" -type f -name "reset_om.sh")" "${GOLDEN_MOUNT_PATH}"; then
        echo "copy reset_om.sh failed, please check"
        return 1
    fi

    if [[ ! -d "${GOLDEN_WHITEBOX_CONFIG_PATH}" ]]; then
        mkdir "${GOLDEN_WHITEBOX_CONFIG_PATH}"
    fi

    if ! copy_whitebox_config "${INPUT_WHITE_BOX_CONFIG_PATH}" "${GOLDEN_WHITEBOX_CONFIG_PATH}"; then
        echo "cp ${INPUT_WHITE_BOX_CONFIG_PATH} to p1 part failed, please check"
        return 1
    fi

    # 删除pki证书
    if [[ -L "${PKI_CERT_PATH}" ]]; then
        unlink "${PKI_CERT_PATH}"
    fi
    if [[ -d "${PKI_CERT_PATH}" ]]; then
        rm -rf "${PKI_CERT_PATH}"
    fi

    if ! umount "${P1_DEV_NAME}"; then
        echo "umount ${P1_DEV_NAME} failed, please check"
        return 1
    fi

    if ! blockdev --setro "${P1_DEV_NAME}"; then
        echo "blockdev --setro ${P1_DEV_NAME} failed, please check"
        return 1
    fi 

    return 0
}

function smooth_whitebox()
{
    # 停止OM所有服务
    for ((i = 0; i < "${#OM_SERVICES[@]}"; i++)); do
        echo "stop ${OM_SERVICES[i]} service"
        if ! systemctl stop "${OM_SERVICES[i]}"; then
            echo "stop ${OM_SERVICES[i]} service failed, please check";
            return 1
        fi
    done
    echo "stop OM service success"

    # 替换OM工作目录的白牌配置，白牌化
    if ! copy_whitebox_config "${INPUT_WHITE_BOX_CONFIG_PATH}" "${OM_WHITEBOX_PATH}"; then
        echo "replace om whitebox config failed, please check"
        return 1
    fi
    if ! chown -hR "$(id "nobody" -u)":"$(id "nobody" -g)" "${OM_WHITEBOX_PATH}"; then
        echo "chown om work dir failed, please check"
        return 1
    fi
    echo "replace om whitebox config success"

    if check_boot_from_emmc; then
        # 将白牌配置、软件包拷贝至p1分区
        if ! copy_files_to_p1_part; then
            echo "copy files to p1 partition failed, please check"
            return 1
        fi
        echo "backup whitebox config to p1 partition success"
    fi

    # 保存白牌配置
    if [[ -d "${WHITEBOX_BACK_DIR}" ]]; then
        if ! chattr -i -R "${WHITEBOX_BACK_DIR}"; then
            echo "chattr -i -R ${WHITEBOX_BACK_DIR} failed, please check"
            return 1
        fi

        if ! rm -r "${WHITEBOX_BACK_DIR}"; then
            echo "clear whitebox config dir ${WHITEBOX_BACK_DIR} failed, please check"
            return 1
        fi
    fi

    if ! mkdir "${WHITEBOX_BACK_DIR}"; then
        echo "create ${WHITEBOX_BACK_DIR} failed, please check"
        return 1
    fi

    if ! copy_whitebox_config "${INPUT_WHITE_BOX_CONFIG_PATH}" "${WHITEBOX_BACK_DIR}"; then
        echo "save ${WHITEBOX_BACK_DIR} failed, please check"
        return 1
    fi

    if ! chattr +i -R "${WHITEBOX_BACK_DIR}"; then
        echo "chattr -i -R ${WHITEBOX_BACK_DIR} failed, please check"
        return 1
    fi
    echo "save whitebox config success"

    # 重启OM所有服务
    for ((i = 0; i < "${#OM_SERVICES[@]}"; i++)); do
        echo "restart ${OM_SERVICES[i]} service"
        if ! systemctl start "${OM_SERVICES[i]}"; then
            echo "start ${OM_SERVICES[i]} service failed, please check";
            return 1
        fi
    done
    echo "restart OM service success"

    return 0
}


function main()
{
    # 检查参数与白牌环境
    if ! check_param_env "$@"; then
        echo "check param failed, whitebox process terminate."
        return 1
    fi
    echo "check whitebox env success, smooth whitebox executing...."

    # 执行白牌化
    smooth_whitebox
    local ret=$?
    rm -rf  "${PACKAGE_UNZIP_PATH}"
    if [[ "${ret}" -ne 0 ]]; then
        echo "smooth whitebox failed, whitebox process terminate."
        return 1
    fi
    echo "whitebox process execute success, OM service has been restarted."

    return 0
}

echo  -e "\n####################### begin to execute whitebox #####################################\n"
main "$@"
echo  -e "\n####################### finish execute whitebox! #########################################\n"
